@@ -12,9 +12,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  * (C) Copyright 2004 Red Hat, Inc.
  */
@@ -22,11 +22,12 @@
 #ifndef VPN_PASSWORD_DIALOG_H
 #define VPN_PASSWORD_DIALOG_H
 
-#include "applet.h"
+#include <glib.h>
+#include <dbus/dbus-glib.h>
+#include <nm-settings.h>
 
-GSList *nma_vpn_request_password (NMApplet *applet, 
-				   const char *name, 
-				   const char *service, 
-				   gboolean retry);
+gboolean nma_vpn_request_password (NMExportedConnection *exported,
+                                   gboolean retry,
+                                   DBusGMethodInvocation *context);
 
 #endif
